﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.NetworkInformation;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Timers;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace WpfApp9
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }
        //通过IsNetworkAlive方法，来获取电脑的联网状态
        [DllImport("sensapi.dll", SetLastError = true)]
        private static extern bool IsNetworkAlive(out int connectionDescription);
        //通过InternetGetConnectedState方法，来获取电脑的联网状态
        [DllImport("winInet.dll")]
        private static extern bool InternetGetConnectedState(ref IntPtr dwFlag, int dwReserved);

        private const int LanNetworkConnectedFlag = 1;
        private void ButtonBase_OnClick(object sender, RoutedEventArgs e)
        {
            var outputDetail = $"通过IsNetworkAlive，检测网络连接。。。\r\n";
            var isNetworkConnected = IsNetworkAlive(out int flags);
            int errCode = Marshal.GetLastWin32Error();
            if (errCode != 0)
            {
                outputDetail += $"IsNetworkAlive函数,遇到异常，code:{errCode}\r\n";
            }

            //IsNetworkAlive检测到是局域网，则使用ping确认是否有网
            if (flags == LanNetworkConnectedFlag)
            {
                outputDetail += $"通过IsNetworkAlive检测到是局域网\r\n";
                var connectedStr = isNetworkConnected ? "已连接" : "断网";
                outputDetail += $"通过IsNetworkAlive，检测网络连接状态:{connectedStr}\r\n";

                outputDetail += $"通过InternetGetConnectedState，检测网络连接。。。\r\n";
                var dwFlag = new IntPtr();
                isNetworkConnected = InternetGetConnectedState(ref dwFlag, 0);
                errCode = Marshal.GetLastWin32Error();
                if (errCode != 0)
                {
                    outputDetail += $"InternetGetConnectedState函数,遇到异常，code:{errCode}\r\n";
                }
                connectedStr = isNetworkConnected ? "已连接" : "断网";
                outputDetail += $"通过InternetGetConnectedState，检测网络连接状态:{connectedStr}\r\n";

                if (!isNetworkConnected)
                {
                    outputDetail += $"通过Ping，检测网络连接。。。\r\n";
                    isNetworkConnected = NetworkUtility.PingServer("easinote.seewo.com");

                    connectedStr = isNetworkConnected ? "已连接" : "断网";
                    outputDetail += $"通过Ping，检测网络连接状态:{connectedStr}\r\n";
                }
            }
            else
            {
                outputDetail += $"通过IsNetworkAlive检测到，不是局域网\r\n";
                var connectedStr = isNetworkConnected ? "已连接" : "断网";
                outputDetail += $"通过IsNetworkAlive，检测网络连接状态:{connectedStr}\r\n";
            }

            outputDetail = (isNetworkConnected ? "已连接" : "断网") + "，详细：\r\n" + outputDetail;
            NetworkStateTextBlock.Text = outputDetail;
        }
    }
    public static class NetworkUtility
    {
        //使用主机名/域名的方式来检测网络的连通性
        public static bool PingServer(string host)
        {
            try
            {
                using (var ping = new Ping())
                {
                    //ping给定的host，超时时间为1s
                    var reply = ping.Send(host, 1000);
                    return reply != null && reply.Status == IPStatus.Success;
                }
            }
            catch (Exception ex)
            {
                //ping不通记log，返回false
                return false;
            }
        }

        //请求DNS返回域名的IP，如果没有错误，说明网络连通
        public static bool DnsLookup(string host)
        {
            try
            {
                Dns.GetHostEntry(host);
                return true;
            }
            catch (Exception ex)
            {
                return false;
            }
        }
    }
}
